# -*- coding: utf-8 -*-
# Generated by Django 1.9.4 on 2017-04-25 09:25
from __future__ import unicode_literals

from django.db import migrations


def createParsers(apps, schema_editor):
    ScriptParameter = apps.get_model('wooey', 'ScriptParameter')
    ScriptParser = apps.get_model('wooey', 'ScriptParser')

    for param in ScriptParameter.objects.all():
        script_version = param.script_version.last()
        parser = list(ScriptParser.objects.filter(script_version__script=script_version.script))
        if parser:
            parser = parser[-1]
        else:
            parser = ScriptParser(
                script_version=script_version,
                name='',
            )
            parser.save()
        param.parser = parser
        param.save()


class Migration(migrations.Migration):

    dependencies = [
        ('wooey', '0028_add_script_subparser'),
    ]

    operations = [
        migrations.RunPython(createParsers),
    ]



import sys

from django import get_version
from distutils.version import StrictVersion
DJANGO_VERSION = StrictVersion(get_version())
DJ21 = StrictVersion('2.1')
DJ20 = StrictVersion('2.0')
DJ111 = StrictVersion('1.11')
DJ110 = StrictVersion('1.10')
DJ19 = StrictVersion('1.9')
DJ18 = StrictVersion('1.8')
DJ17 = StrictVersion('1.7')
DJ16 = StrictVersion('1.6')

PY_FULL_VERSION = StrictVersion('{}.{}.{}'.format(sys.version_info.major, sys.version_info.minor, sys.version_info.micro))
PY_MINOR_VERSION = StrictVersion('{}.{}'.format(sys.version_info.major, sys.version_info.minor))
PY34 = StrictVersion('3.4')
PY343 = StrictVersion('3.4.3')
PY33 = StrictVersion('3.3')
PY336 = StrictVersion('3.3.6')



# -*- coding: utf-8 -*-
# Generated by Django 1.9.7 on 2016-07-02 14:26
from __future__ import unicode_literals

from django.db import migrations


def update_script_versions(apps, schema_editor):
    ScriptParameter = apps.get_model('wooey', 'ScriptParameter')

    for script_parameter in ScriptParameter.objects.all():
        script_parameter.script_versions.add(script_parameter.script_version)


class Migration(migrations.Migration):

    dependencies = [
        ('wooey', '0023_script_parameter_m2m'),
    ]

    operations = [
        migrations.RunPython(update_script_versions),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.4 on 2016-05-08 12:23
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('wooey', '0021_fix_verbose_names'),
    ]

    operations = [
        migrations.AddField(
            model_name='scriptparameter',
            name='collapse_arguments',
            field=models.BooleanField(default=True, help_text='Collapse separate inputs to a given argument to a single input (ie: --arg 1 --arg 2 becomes --arg 1 2)'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.13 on 2018-06-23 20:41
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('wooey', '0039_wooey_widgets'),
    ]

    operations = [
        migrations.AddField(
            model_name='wooeywidget',
            name='widget_class',
            field=models.CharField(blank=True, help_text='Widget class to use (e.g. django.forms.TextInput, defaults to Form Field on Script Parameter model if blank).', max_length=50, null=True),
        ),
        migrations.AlterField(
            model_name='scriptparameter',
            name='input_type',
            field=models.CharField(help_text='The python type expected by the script (e.g. boolean, integer, file).', max_length=255),
        ),
        migrations.AlterField(
            model_name='wooeywidget',
            name='input_attributes',
            field=models.TextField(blank=True, help_text='Extra attributes to the input field. The extra attributes MUST be specified like key="value" (e.g. type="date").', null=True, verbose_name='Input Widget Extra Attributes'),
        ),
    ]



import argparse
import sys
from numpy import mgrid, linspace, zeros, copy, multiply, add
from matplotlib import pyplot as plt

parser = argparse.ArgumentParser(description="Something")
parser.add_argument('height', help='The height of the image.', type=int, default=400)
parser.add_argument('width', help='the width of the image.', type=int, default=400)
parser.add_argument('xmin', help='The minimum region to compute.', type=float, default=-2)
parser.add_argument('xmax', help='The maximum region to compute.', type=float, default=0.5)
parser.add_argument('ymin', help='The minimum region to compute.', type=float, default=-1.25)
parser.add_argument('ymax', help='The maximum region to compute.', type=float, default=1.25)


def mandel(n, m, itermax, xmin, xmax, ymin, ymax):
    # from https://thesamovar.wordpress.com/2009/03/22/fast-fractals-with-python-and-numpy/
    ix, iy = mgrid[0:n, 0:m]
    x = linspace(xmin, xmax, n)[ix]
    y = linspace(ymin, ymax, m)[iy]
    c = x+complex(0, 1)*y
    del x, y
    img = zeros(c.shape, dtype=int)
    ix.shape = n*m
    iy.shape = n*m
    c.shape = n*m
    z = copy(c)
    for i in xrange(itermax):
        if not len(z):
            break
        multiply(z, z, z)
        add(z, c, z)
        rem = abs(z)>2.0
        img[ix[rem], iy[rem]] = i+1
        rem = -rem
        z = z[rem]
        ix, iy = ix[rem], iy[rem]
        c = c[rem]

    plt.imshow(img)
    plt.savefig('fractal.png')


def limit(value, min, max, default):
    if value < min or value > max:
        return default
    return value

if __name__ == '__main__':
    args = parser.parse_args()
    height = limit(args.height, 1, 2000, 400)
    width = limit(args.width, 1, 2000, 400)
    mandel(args.height, args.width, 100, args.xmin, args.xmax, args.ymin, args.ymax)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.12 on 2018-04-08 20:12
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('wooey', '0037_repeat_migration_spg_sp_script_version'),
    ]

    operations = [
        migrations.AlterField(
            model_name='wooeyjob',
            name='status',
            field=models.CharField(choices=[('completed', 'Completed'), ('deleted', 'Deleted'), ('FAILURE', 'Failed'), ('running', 'Running'), ('submitted', 'Submitted')], default='submitted', max_length=255),
        ),
    ]



from __future__ import absolute_import
import os
import subprocess
import sys
import tarfile
import tempfile
import traceback
import zipfile
from threading import Thread

import six
from django.utils.text import get_valid_filename
from django.core.files import File
from django.conf import settings

from celery import Task
from celery import app
from celery.schedules import crontab
from celery.signals import worker_process_init

from .backend import utils
from . import settings as wooey_settings

try:
    from Queue import Empty, Queue
except ImportError:
    from queue import Empty, Queue  # python 3.x

ON_POSIX = 'posix' in sys.builtin_module_names

celery_app = app.app_or_default()


def enqueue_output(out, q):
    for line in iter(out.readline, b''):
        q.put(line.decode('utf-8'))
    try:
        out.close()
    except IOError:
        pass


def output_monitor_queue(queue, out):
    p = Thread(target=enqueue_output, args=(out, queue))
    p.start()
    return p


def update_from_output_queue(queue, out):
    lines = []
    while True:
        try:
            line = queue.get_nowait()
            lines.append(line)
        except Empty:
            break

    out += ''.join(map(str, lines))
    return out


@worker_process_init.connect
def configure_workers(*args, **kwargs):
    # this sets up Django on nodes started by the worker daemon.
    import django
    django.setup()


class WooeyTask(Task):
    pass

    # def after_return(self, status, retval, task_id, args, kwargs, einfo):
    #     job, created = WooeyJob.objects.get_or_create(wooey_celery_id=task_id)
    #     job.content_type.wooey_celery_state = status
    #     job.save()


def get_latest_script(script_version):
    """Downloads the latest script version to the local storage.

    :param script_version: :py:class:`~wooey.models.core.ScriptVersion`
    :return: boolean
        Returns true if a new version was downloaded.
    """
    script_path = script_version.script_path
    local_storage = utils.get_storage(local=True)
    script_exists = local_storage.exists(script_path.name)
    if not script_exists:
        local_storage.save(script_path.name, script_path.file)
        return True
    else:
        # If script exists, make sure the version is valid, otherwise fetch a new one
        script_contents = local_storage.open(script_path.name).read()
        script_checksum = utils.get_checksum(buff=script_contents)
        if script_checksum != script_version.checksum:
            tf = tempfile.TemporaryFile()
            with tf:
                tf.write(script_contents)
                tf.seek(0)
                local_storage.delete(script_path.name)
                local_storage.save(script_path.name, tf)
                return True
    return False


@celery_app.task(base=WooeyTask)
def submit_script(**kwargs):
    job_id = kwargs.pop('wooey_job')
    resubmit = kwargs.pop('wooey_resubmit', False)
    from .models import WooeyJob, UserFile
    job = WooeyJob.objects.get(pk=job_id)

    command = utils.get_job_commands(job=job)
    if resubmit:
        # clone ourselves, setting pk=None seems hackish but it works
        job.pk = None

    # This is where the script works from -- it is what is after the media_root since that may change between
    # setups/where our user uploads are stored.
    cwd = job.get_output_path()

    abscwd = os.path.abspath(os.path.join(settings.MEDIA_ROOT, cwd))
    job.command = ' '.join(command)
    job.save_path = cwd

    utils.mkdirs(abscwd)
    # make sure we have the script, otherwise download it. This can happen if we have an ephemeral file system or are
    # executing jobs on a worker node.
    get_latest_script(job.script_version)


    job.status = WooeyJob.RUNNING
    job.save()

    stdout, stderr = '', ''
    proc = subprocess.Popen(command, stdout=subprocess.PIPE, stderr=subprocess.PIPE, cwd=abscwd, bufsize=0)

    # We need to use subprocesses to capture the IO, otherwise they will block one another
    # i.e. a check against stderr will sit waiting on stderr before returning
    # we use Queues to communicate
    qout, qerr = Queue(), Queue()
    pout = output_monitor_queue(qout, proc.stdout)
    perr = output_monitor_queue(qerr, proc.stderr)

    prev_std = None

    def check_output(job, stdout, stderr, prev_std):
        # Check for updates from either (non-blocking)
        stdout = update_from_output_queue(qout, stdout)
        stderr = update_from_output_queue(qerr, stderr)

        # If there are changes, update the db
        if (stdout, stderr) != prev_std:
            job.update_realtime(stdout=stdout, stderr=stderr)
            prev_std = (stdout, stderr)

        return stdout, stderr, prev_std

    # Loop until the process is complete + both stdout/stderr have EOFd
    while proc.poll() is None or pout.is_alive() or perr.is_alive():
        stdout, stderr, prev_std = check_output(job, stdout, stderr, prev_std)

    # Catch any remaining output
    try:
        proc.stdout.flush()
    except ValueError:  # Handle if stdout is closed
        pass
    stdout, stderr, prev_std = check_output(job, stdout, stderr, prev_std)

    # tar/zip up the generated content for bulk downloads
    def get_valid_file(cwd, name, ext):
        out = os.path.join(cwd, name)
        index = 0
        while os.path.exists(six.u('{}.{}').format(out, ext)):
            index += 1
            out = os.path.join(cwd, six.u('{}_{}').format(name, index))
        return six.u('{}.{}').format(out, ext)

    # fetch the job again in case the database connection was lost during the job or something else changed.
    job = WooeyJob.objects.get(pk=job_id)
    # if there are files generated, make zip/tar files for download
    if len(os.listdir(abscwd)):
        tar_out = get_valid_file(abscwd, get_valid_filename(job.job_name), 'tar.gz')
        tar = tarfile.open(tar_out, "w:gz")
        tar_name = os.path.splitext(os.path.splitext(os.path.split(tar_out)[1])[0])[0]
        tar.add(abscwd, arcname=tar_name)
        tar.close()

        zip_out = get_valid_file(abscwd, get_valid_filename(job.job_name), 'zip')
        zip = zipfile.ZipFile(zip_out, "w")
        arcname = os.path.splitext(os.path.split(zip_out)[1])[0]
        zip.write(abscwd, arcname=arcname)
        base_dir = os.path.split(zip_out)[0]
        for root, folders, filenames in os.walk(base_dir):
            for filename in filenames:
                path = os.path.join(root, filename)
                archive_name = path.replace(base_dir, '')
                if archive_name.startswith(os.path.sep):
                    archive_name = archive_name.replace(os.path.sep, '', 1)
                archive_name = os.path.join(arcname, archive_name)
                if path == tar_out:
                    continue
                if path == zip_out:
                    continue
                try:
                    zip.write(path, arcname=archive_name)
                except:
                    stderr = '{}\n{}'.format(stderr, traceback.format_exc())
        try:
            zip.close()
        except:
            stderr = '{}\n{}'.format(stderr, traceback.format_exc())

        # save all the files generated as well to our default storage for ephemeral storage setups
        if wooey_settings.WOOEY_EPHEMERAL_FILES:
            for root, folders, files in os.walk(abscwd):
                for filename in files:
                    filepath = os.path.join(root, filename)
                    s3path = os.path.join(root[root.find(cwd):], filename)
                    remote = utils.get_storage(local=False)
                    exists = remote.exists(s3path)
                    filesize = remote.size(s3path) if exists else 0
                    if not exists or (exists and filesize == 0):
                        if exists:
                            remote.delete(s3path)
                        remote.save(s3path, File(open(filepath, 'rb')))
    utils.create_job_fileinfo(job)

    job.stdout = stdout
    job.stderr = stderr
    job.status = WooeyJob.COMPLETED
    job.update_realtime(delete=True)
    job.save()

    return (stdout, stderr)


@celery_app.task(base=WooeyTask)
def cleanup_wooey_jobs(**kwargs):
    from django.utils import timezone
    from .models import WooeyJob

    cleanup_settings = wooey_settings.WOOEY_JOB_EXPIRATION
    anon_settings = cleanup_settings.get('anonymous')
    now = timezone.now()
    if anon_settings:
        WooeyJob.objects.filter(user=None, created_date__lte=now-anon_settings).delete()
    user_settings = cleanup_settings.get('user')
    if user_settings:
        WooeyJob.objects.filter(user__isnull=False, created_date__lte=now-user_settings).delete()


@celery_app.task(base=WooeyTask)
def cleanup_dead_jobs():
    """
    This cleans up jobs that have been marked as ran, but are not queue'd in celery. It is meant
    to cleanup jobs that have been lost due to a server crash or some other reason a job is
    in limbo.
    """
    from .models import WooeyJob

    # Get active tasks from Celery
    inspect = celery_app.control.inspect()
    worker_info = inspect.active()

    # If we cannot connect to the workers, we do not know if the tasks are running or not, so
    # we cannot mark them as dead
    if not worker_info:
        return

    active_tasks = {task['id'] for worker, tasks in six.iteritems(worker_info) for task in tasks}

    # find jobs that are marked as running but not present in celery's active tasks
    active_jobs = WooeyJob.objects.filter(status=WooeyJob.RUNNING)
    to_disable = set()
    for job in active_jobs:
        if job.celery_id not in active_tasks:
            to_disable.add(job.pk)

    WooeyJob.objects.filter(pk__in=to_disable).update(status=WooeyJob.FAILED)


celery_app.conf.beat_schedule.update({
    'cleanup-old-jobs': {
        'task': 'wooey.tasks.cleanup_wooey_jobs',
        'schedule': crontab(hour=0, minute=0),  # cleanup at midnight each day
    },
    'cleanup-dead-jobs': {
        'task': 'wooey.tasks.cleanup_dead_jobs',
        'schedule': crontab(minute='*/10'),  # run every 6 minutes
    }
})



#!/usr/bin/env python
__author__ = 'chris'
import argparse
import sys
import math
from matplotlib import pyplot as plt
import numpy as np

parser = argparse.ArgumentParser(description="This will plot a gaussian distribution with the given parameters.")
parser.add_argument('--mean', help='The mean of the gaussian.', type=float, required=True)
parser.add_argument('--std', help='The standard deviation (width) of the gaussian.', type=float, required=True)


def main():
    args = parser.parse_args()
    u = args.mean
    s = abs(args.std)
    variance = s**2
    amplitude = 1/(s*math.sqrt(2*math.pi))
    fit = lambda x: [amplitude*math.exp((-1*(xi-u)**2)/(2*variance)) for xi in x]
    # plot +- 4 standard deviations
    X = np.linspace(u-4*s, u+4*s, 100)
    Y = fit(X)
    plt.plot(X, Y)
    plt.title('Gaussian distribution with mu={0:.2f}, sigma={1:.2f}'.format(u, s))
    plt.savefig('gaussian.png')

if __name__ == "__main__":
    sys.exit(main())



# -*- coding: utf-8 -*-
# Generated by Django 1.11.11 on 2018-03-28 09:50
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('wooey', '0036_make_scriptparameter_jsonfield'),
    ]

    operations = [
        # These migrations were covered by 0029->0032 but for some reason Django wants to
        # repeat them. This should be a pointless change.
        migrations.AlterField(
            model_name='scriptparametergroup',
            name='script_version',
            field=models.ManyToManyField(to='wooey.ScriptVersion'),
        ),
        migrations.AlterField(
            model_name='scriptparser',
            name='script_version',
            field=models.ManyToManyField(to='wooey.ScriptVersion'),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations

def gen_checksums(apps, schema_editor):
    WooeyFile = apps.get_model('wooey', 'WooeyFile')
    from ..backend.utils import get_checksum
    for obj in WooeyFile.objects.all():
        try:
            obj.checksum = get_checksum(path=obj.filepath.path)
            obj.save()
        except IOError:
            print(obj.filepath, 'not found')


class Migration(migrations.Migration):

    dependencies = [
        ('wooey', '0016_wooeyfile_checksum'),
    ]

    operations = [
        migrations.RunPython(gen_checksums),
    ]



import traceback
import sys

try:
    from django.apps import AppConfig
except ImportError:
    AppConfig = object


class WooeyConfig(AppConfig):
    name = 'wooey'
    verbose_name = 'Wooey'

    def ready(self):
        from . import signals



import factory
import six
from django.contrib.auth import get_user_model

from ..models import Script, ScriptGroup, WooeyJob, WooeyWidget
from . import utils as test_utils


class ScriptGroupFactory(factory.DjangoModelFactory):
    class Meta:
        model = ScriptGroup

    group_name = 'test group'
    group_description = 'test desc'


class ScriptParameterGroupFactory(factory.DjangoModelFactory):
    class Meta:
        model = 'wooey.ScriptParameterGroup'


class ScriptParameterFactory(factory.DjangoModelFactory):
    class Meta:
        model = 'wooey.ScriptParameter'

    script_param = 'script-param'
    is_output = False
    parameter_group = factory.SubFactory('wooey.tests.factories.ScriptParameterGroupFactory')
    parser = factory.SubFactory('wooey.tests.factories.ScriptParserFactory')


class ScriptParserFactory(factory.DjangoModelFactory):
    class Meta:
        model = 'wooey.ScriptParser'


class ScriptFactory(factory.DjangoModelFactory):
    class Meta:
        model = Script

    script_name = 'test script'
    script_group = factory.SubFactory(ScriptGroupFactory)
    script_description = 'test script desc'


class UserFactory(factory.DjangoModelFactory):
    class Meta:
        model = get_user_model()
        django_get_or_create = ('username',)

    username = 'user'
    email = 'a@a.com'
    password = 'testuser'


class BaseJobFactory(factory.DjangoModelFactory):
    class Meta:
        model = WooeyJob
    job_name = six.u('\xd0\xb9\xd1\x86\xd1\x83')
    job_description = six.u('\xd0\xb9\xd1\x86\xd1\x83\xd0\xb5\xd0\xba\xd0\xb5')


class WooeyWidgetFactory(factory.DjangoModelFactory):
    class Meta:
        model = WooeyWidget

    name = 'test widget'


def generate_script(script_path, script_name=None):
    new_file = test_utils.save_script_path(script_path)
    from ..backend import utils
    res = utils.add_wooey_script(script_name=script_name, script_path=new_file, group=None)
    return res['script']


def generate_job(script_version):
    return BaseJobFactory(script_version=script_version)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.11 on 2018-03-28 09:48
from __future__ import unicode_literals

from django.db import migrations, models
import jsonfield.fields


class Migration(migrations.Migration):

    dependencies = [
        ('wooey', '0035_populate-jsonfield'),
    ]

    operations = [
        migrations.AlterField(
            model_name='scriptparameter',
            name='default',
            field=jsonfield.fields.JSONCharField(blank=True, max_length=255, null=True),
        ),
    ]



# TODO: Test for viewing a user's job as an anonymous user (fail case)

import json

from django.test import TestCase, RequestFactory
from django.contrib.auth import get_user_model
from django.contrib.auth.models import AnonymousUser
from django.http import Http404
from nose.tools import raises

from . import (
    config,
    factories,
    mixins,
    utils as test_utils,
)
from ..backend import utils
from ..django_compat import reverse
from ..views import wooey_celery
from .. import views as wooey_views
from .. import models
from .. import settings


User = get_user_model()


def load_JSON_dict(d):
    return json.loads(d.decode('utf-8'))


class CeleryViews(mixins.ScriptFactoryMixin, mixins.FileCleanupMixin, TestCase):
    def setUp(self):
        super(CeleryViews, self).setUp()
        self.factory = RequestFactory()
        self.user = factories.UserFactory()
        # the test server doesn't have celery running
        settings.WOOEY_CELERY = False

    def test_celery_results(self):
        request = self.factory.get(reverse('wooey:all_queues_json'))
        request.user = self.user
        response = wooey_celery.all_queues_json(request)
        d = response.content.decode("utf-8")
        self.assertEqual(
            {
                u'items': {u'global': [], u'results': [], u'user': []},
                u'totals': {u'global': 0, u'results': 0, u'user': 0}
            },
            json.loads(d)
        )

        job = factories.generate_job(self.translate_script)
        job.save()
        response = wooey_celery.all_queues_json(request)
        d = json.loads(response.content.decode("utf-8"))
        self.assertEqual(1, d['totals']['global'])

        job.user = self.user
        job.status = models.WooeyJob.RUNNING
        job.save()
        response = wooey_celery.all_queues_json(request)
        d = json.loads(response.content.decode("utf-8"))
        # we now are logged in, make sure the job appears under the user jobs
        self.assertEqual(1, d['totals']['user'])

        user = AnonymousUser()
        request.user = user
        response = wooey_celery.all_queues_json(request)
        d = json.loads(response.content.decode("utf-8"))
        # test empty response since anonymous users should not see users jobs
        self.assertEqual(d['items']['results'], [])
        self.assertEqual(d['items']['user'], [])

    def test_celery_commands(self):
        job = factories.generate_job(self.translate_script)
        job.user = self.user
        job.save()
        celery_command = {'celery-command': ['delete'], 'job-id': [job.pk]}
        # test that we cannot modify a users script
        request = self.factory.post(reverse('wooey:celery_task_command'),
                                    celery_command)
        anon = AnonymousUser()
        request.user = anon
        response = wooey_celery.celery_task_command(request)
        d = response.content.decode("utf-8")
        self.assertFalse(json.loads(d).get('valid'))

        # test a nonsense command
        celery_command.update({'celery-command': ['thisshouldfail']})
        response = wooey_celery.celery_task_command(request)
        d = response.content.decode("utf-8")
        self.assertFalse(json.loads(d).get('valid'))

        # test that the user can interact with it
        # the stop command will break, so currently untested here until I figure it out
        for i in ['resubmit', 'rerun', 'delete']:
            celery_command.update({'celery-command': [i]})
            request = self.factory.post(
                reverse('wooey:celery_task_command'),
                celery_command,
            )
            request.user = self.user
            response = wooey_celery.celery_task_command(request)
            d = response.content.decode("utf-8")
            self.assertTrue(json.loads(d).get('valid'))

    def test_celery_task_view(self):
        job = factories.generate_job(self.translate_script)
        job.user = self.user
        job.save()

        # test that an anonymous user cannot view a user's job
        view = wooey_celery.JobView.as_view()
        request = self.factory.get(reverse('wooey:celery_results', kwargs={'job_id': job.pk}))
        request.user = AnonymousUser()
        response = view(request, job_id=job.pk)
        response.render()
        self.assertIn('job_error', response.context_data)
        self.assertNotIn('job_info', response.context_data)

        # test the user can view the job
        request.user = self.user
        response = view(request, job_id=job.pk)
        response.render()
        self.assertNotIn('job_error', response.context_data)
        self.assertIn('job_info', response.context_data)

    @raises(Http404)
    def test_celery_nonexistent_task(self):
        # test request for non-existent job, should raise 404
        view = wooey_celery.JobView.as_view()
        request = self.factory.get(reverse('wooey:celery_results', kwargs={'job_id': '-1'}))
        response = view(request, job_id=-1)
        response.render()


class WooeyViews(mixins.ScriptFactoryMixin, mixins.FileCleanupMixin, TestCase):
    def setUp(self):
        super(WooeyViews, self).setUp()
        self.factory = RequestFactory()
        self.script_view_func = wooey_views.WooeyScriptView.as_view()
        self.json_view_func = wooey_views.WooeyScriptJSON.as_view()
        self.user = factories.UserFactory()
        # the test server doesn't have celery running
        settings.WOOEY_CELERY = False

    def test_multiple_choice_clone(self):
        from ..backend import utils
        script_version = self.choice_script
        script = script_version.script
        choices = ['2', '1', '3']
        choice_param = 'two_choices'
        job = utils.create_wooey_job(script_version_pk=script_version.pk, data={'job_name': 'abc', choice_param: choices, 'wooey_type': script_version.pk})
        request = self.factory.post(
            reverse(
                'wooey:wooey_script_clone',
                kwargs={
                    'slug': job.script_version.script.slug,
                    'job_id': job.pk
                }
            ),
            data={
                'wooey_type': script_version.pk,
            }
        )
        request.user = AnonymousUser()
        response = self.json_view_func(request, pk=script.pk, job_id=job.pk)
        self.assertEqual(response.status_code, 200)

    def test_clone_job_into_specific_version(self):
        # Test we can clone a job into a previous script's version

        # Create a job using version2
        job = factories.generate_job(self.version2_script)

        # Get a job using version 1
        job_kwargs = {
            'slug': self.version1_script.script.slug,
            'script_version': self.version1_script.script_version,
            'script_iteration': self.version1_script.script_iteration,
            'job_id': job.pk,
        }
        request = self.factory.get(
            reverse(
                'wooey:wooey_script',
                kwargs=job_kwargs
            ),
        )
        request.user = AnonymousUser()
        response = self.script_view_func(request, pk=self.version1_script.script.pk, **job_kwargs)
        self.assertEqual(response.status_code, 200)

        # Test that version1 was returned
        context = response.resolve_context(response.context_data)
        self.assertEqual(context['form']['wooey_form']['wooey_type'].value(), self.version1_script.pk)


    def test_multiple_choice(self):
        script_version = self.choice_script
        script = script_version.script
        two_choices_slug = test_utils.get_subparser_form_slug(script_version, 'two_choices')
        url = reverse('wooey:wooey_script', kwargs={'slug': script.slug})
        data = {
            'job_name': 'abc',
            'wooey_type': script_version.pk,
            'wooey_parser': script_version.scriptparser_set.first().pk,
            two_choices_slug: ['2', '1', '3']}
        filecount = 0
        for i, v in config.SCRIPT_DATA['choices']['files'].items():
            slug = test_utils.get_subparser_form_slug(script_version, i)
            data[slug] = v
            filecount += len(v)
        request = self.factory.post(url, data=data)
        request.user = self.user
        response = self.json_view_func(request)
        d = load_JSON_dict(response.content)
        self.assertTrue(d['valid'], d)
        self.assertEqual(sum([len(request.FILES.getlist(i)) for i in request.FILES.keys()]), filecount)

        # test submitting this in the 'currently' field
        job = models.WooeyJob.objects.latest('created_date')
        files = [i.value.name for i in job.get_parameters() if i.parameter.slug == 'multiple_file_choices']

        data['multiple_file_choices'] = files
        request = self.factory.post(url, data=data)
        request.user = self.user
        response = self.json_view_func(request)
        self.assertEqual(response.status_code, 200)
        d = load_JSON_dict(response.content)
        self.assertTrue(d['valid'], d)

        # check the files are actually with the new model
        job = models.WooeyJob.objects.latest('created_date')
        new_files = [i.value.url for i in job.get_parameters() if i.parameter.slug == 'multiple_file_choices']
        self.assertEqual(len(new_files), len(files))


    def test_form_groups(self):
        # Make sure forms groups work to validate
        script_version = self.without_args
        forms = utils.get_form_groups(script_version=self.without_args)
        data = {}
        data.update(config.SCRIPT_DATA['without_args'].get('data'))
        wooey_form = forms['wooey_form']
        data.update(wooey_form.initial)
        subparser = list(forms['parsers'].keys())[0][0]
        data['wooey_parser'] = subparser

        url = reverse('wooey:wooey_script', kwargs={'slug': script_version.script.slug})
        request = self.factory.post(url, data=data)
        user = factories.UserFactory()
        request.user = user
        response = self.json_view_func(request)
        d = load_JSON_dict(response.content)
        self.assertTrue(d['valid'], d)


    def test_job_view_permissions(self):
        # Make sure users cannot see jobs from other users
        job = factories.generate_job(self.translate_script)
        url = reverse('wooey:celery_results', kwargs={'job_id': job.pk})

        # Make a new user
        user1 = User(username='wooey')
        user1.save()
        user2 = User(username='wooey2')
        user2.save()
        request = self.factory.get(url)
        request.user = user1
        view = wooey_celery.JobView.as_view()
        response = view(request, job_id=job.pk)
        self.assertEqual(response.status_code, 200)

        job.user = user2
        job.save()

        response = view(request, job_id=job.pk)
        response.render()
        self.assertContains(response, models.WooeyJob.error_messages['invalid_permissions'])

    def test_validates_chosen_subparser(self):
        # Addresses https://github.com/wooey/Wooey/issues/288
        script_version = self.subparser_script
        script = script_version.script
        sp2_slug = test_utils.get_subparser_form_slug(script_version, 'sp2')
        subparser = script_version.scriptparser_set.get(name='subparser2')
        url = reverse('wooey:wooey_script', kwargs={'slug': script.slug})
        data = {
            'job_name': 'abc',
            'wooey_type': script_version.pk,
            'wooey_parser': subparser.pk,
            sp2_slug: [1],
        }
        request = self.factory.post(url, data=data)
        request.user = self.user
        response = self.json_view_func(request)
        d = load_JSON_dict(response.content)
        self.assertTrue(d['valid'], d)

    def test_fails_if_required_subparser_argument_not_used(self):
        # Addresses https://github.com/wooey/Wooey/issues/288
        script_version = self.subparser_script
        script = script_version.script
        subparser = script_version.scriptparser_set.get(name='subparser2')
        url = reverse('wooey:wooey_script', kwargs={'slug': script.slug})
        data = {
            'job_name': 'abc',
            'wooey_type': script_version.pk,
            'wooey_parser': subparser.pk,
        }
        request = self.factory.post(url, data=data)
        request.user = self.user
        response = self.json_view_func(request)
        d = load_JSON_dict(response.content)
        self.assertFalse(d['valid'], d)



from .django_settings import *
from wooey.version import DJANGO_VERSION, DJ110
from django.utils.translation import ugettext_lazy as _

INSTALLED_APPS += (
    # 'corsheaders',
    'wooey',
)

if DJANGO_VERSION < DJ110:
    MIDDLEWARE_CLASSES = list(MIDDLEWARE_CLASSES)
    MIDDLEWARE_CLASSES.append('{{ project_name }}.middleware.ProcessExceptionMiddleware')
    MIDDLEWARE_OBJ = MIDDLEWARE_CLASSES
else:
    # Using Django 1.10 +
    MIDDLEWARE = list(MIDDLEWARE)
    MIDDLEWARE.append('{{ project_name }}.middleware.ProcessExceptionMiddleware')
    MIDDLEWARE_OBJ = MIDDLEWARE

LANGUAGES = [
    ('de', _('German')),
    ('en', _('English')),
    ('fr', _('French')),
    ('ja', _('Japanese')),
    ('nl', _('Dutch')),
    ('zh-hans', _('Simplified Chinese')),
    ('ko', _('Korean')),
]

NEW_MIDDLEWARE = []
for i in MIDDLEWARE_OBJ:
    NEW_MIDDLEWARE.append(i)
    if i == 'django.contrib.sessions.middleware.SessionMiddleware':
        NEW_MIDDLEWARE.append('django.middleware.locale.LocaleMiddleware')

NEW_MIDDLEWARE.append('{{ project_name }}.middleware.ProcessExceptionMiddleware')
if DJANGO_VERSION < DJ110:
    MIDDLEWARE_CLASSES = NEW_MIDDLEWARE
else:
    MIDDLEWARE = NEW_MIDDLEWARE

PROJECT_NAME = "{{ project_name }}"
WOOEY_CELERY_APP_NAME = 'wooey.celery'
WOOEY_CELERY_TASKS = 'wooey.tasks'



from __future__ import unicode_literals
import os

from django.core.files import File
from django.conf import settings

BASE_DIR = os.path.split(__file__)[0]
WOOEY_TEST_SCRIPTS = os.path.join(BASE_DIR, 'scripts')
WOOEY_TEST_DATA = os.path.join(BASE_DIR, 'data')
WOOEY_TEST_REMOTE_STORAGE_DIR = 'remote_storage'
WOOEY_TEST_REMOTE_STORAGE_PATH = os.path.join(BASE_DIR, 'media', 'remote_storage')

# Because forms are input as lists by Django, all attributes here need to be
# list like as well. The MultiValueDict/QueryDict get method assumes a list and
# will take the first element in validation (so for a string like 'ATAT' it will
# provide 'A').
SCRIPT_DATA = {
    'translate':
        {
            'data': {
                'wooey_type': ['1'],
                'job_name': ['abc'],
                'sequence': ['ATATATATATA'],
                'frame': ['+3'],
                'out': ['abc']
            }
        },
    'choices':
        {
            'data': {
                'wooey_type': ['1'],
                'job_name': ['abc'],
                'two_choices': [1, 2],
            },
            'files': {
                'multiple_file_choices': [
                    File(open(os.path.join(WOOEY_TEST_SCRIPTS, 'choices.py'), 'rb')),
                    File(open(os.path.join(WOOEY_TEST_SCRIPTS, 'crop.py'), 'rb'))
                ]
            }
        },
    'without_args':
        {
            'data': {
                'job_name': ['abc'],
            }

        }
}



__author__ = 'chris'
from unittest import TestCase
import subprocess
import os
import shutil
import sys

BASE_DIR = os.path.split(__file__)[0]
WOOEY_SCRIPT_PATH = os.path.join(BASE_DIR, '..', 'scripts', 'wooify')
WOOEY_TEST_PROJECT_NAME = 'wooey_project'
WOOEY_TEST_PROJECT_PATH = os.path.join(BASE_DIR, WOOEY_TEST_PROJECT_NAME)
WOOEY_TEST_PROJECT_MANAGE = os.path.join(WOOEY_TEST_PROJECT_PATH, 'manage.py')
PYTHON_INTERPRETTER = sys.executable if sys.executable else 'python'

env = os.environ
env['DJANGO_SETTINGS_MODULE'] = '{}.settings'.format(WOOEY_TEST_PROJECT_NAME)
env['TESTING'] = 'True'


class TestProject(TestCase):
    def setUp(self):
        os.chdir(BASE_DIR)
        # if old stuff exists, remove it
        if os.path.exists(WOOEY_TEST_PROJECT_PATH):
            shutil.rmtree(WOOEY_TEST_PROJECT_PATH)

    def tearDown(self):
        os.chdir(BASE_DIR)
        if os.path.exists(WOOEY_TEST_PROJECT_PATH):
            shutil.rmtree(WOOEY_TEST_PROJECT_PATH)

    def test_bootstrap(self):
        from wooey.backend import command_line
        sys.argv = [WOOEY_SCRIPT_PATH, '-p', WOOEY_TEST_PROJECT_NAME]
        ret = command_line.bootstrap(env=env, cwd=BASE_DIR)
        self.assertIsNone(ret)
        # test our script is executable from the command line, it will fail with return code of 1 since
        # the project already exists
        proc = subprocess.Popen([PYTHON_INTERPRETTER, WOOEY_SCRIPT_PATH, '-p', WOOEY_TEST_PROJECT_NAME])
        stdout, stderr  = proc.communicate()
        self.assertEqual(proc.returncode, 1, stderr)



# -*- coding: utf-8 -*-
# Generated by Django 1.9.13 on 2018-03-04 23:14
from __future__ import unicode_literals
import json

from django.db import migrations


def populate_default(apps, schema_editor):
    ScriptParameter = apps.get_model('wooey', 'ScriptParameter')
    for obj in ScriptParameter.objects.all():
        try:
            obj.default = json.loads(obj.default)
        except Exception:
            obj.default = obj.default
        obj.save()


def reverse_populate_default(apps, schema_editor):
    pass


class Migration(migrations.Migration):

    dependencies = [
        ('wooey', '0034_update-checksums'),
    ]

    operations = [
        migrations.RunPython(populate_default, reverse_populate_default)
    ]



import mock
import os
from datetime import timedelta

from django.test import TestCase

from wooey import settings as wooey_settings
from wooey.backend.utils import add_wooey_script
from wooey.models import (
    WooeyJob,
)
from wooey.tasks import (
    cleanup_dead_jobs,
    get_latest_script,
)

from . import config, mixins, factories

class TaskTests(mixins.ScriptFactoryMixin, TestCase):

    def test_job_cleanup(self):
        from ..models import WooeyJob
        from ..tasks import cleanup_wooey_jobs
        import time
        anon_job = factories.generate_job(self.translate_script)
        user_job = factories.generate_job(self.translate_script)
        user = factories.UserFactory()
        user_job.user = user
        user_job.save()
        wooey_settings.WOOEY_JOB_EXPIRATION.update({
            'user': timedelta(hours=1),
            'anonymous': timedelta(hours=1),
        })
        cleanup_wooey_jobs()
        self.assertListEqual(list(WooeyJob.objects.all()), [anon_job, user_job])
        time.sleep(0.1)
        wooey_settings.WOOEY_JOB_EXPIRATION.update({
            'user': timedelta(hours=1),
            'anonymous': timedelta(microseconds=1),
        })
        cleanup_wooey_jobs()
        self.assertListEqual(list(WooeyJob.objects.all()), [user_job])

        wooey_settings.WOOEY_JOB_EXPIRATION.update({
            'user': timedelta(microseconds=1),
            'anonymous': timedelta(microseconds=1),
        })

        cleanup_wooey_jobs()
        self.assertListEqual(list(WooeyJob.objects.all()), [])


class TestGetLatestScript(mixins.FileMixin, mixins.ScriptTearDown, TestCase):
    def setUp(self):
        super(TestGetLatestScript, self).setUp()
        script = os.path.join(config.WOOEY_TEST_SCRIPTS, 'versioned_script', 'v1.py')
        with open(script) as o:
            v1 = self.storage.save(self.filename_func('v1.py'), o)
        res = add_wooey_script(script_path=v1, script_name='test_versions')
        self.first_version = self.rename_script(res['script'])

    def rename_script(self, script_version):
        # Because we are on local storage, the script uploaded will already be present, so
        # we rename it to mimic it being absent on a worker node
        new_name = self.storage.save(script_version.script_path.name, script_version.script_path.file)
        script_version.script_path.name = new_name
        script_version.save()
        return script_version

    def test_get_latest_script_loads_initial(self):
        self.assertTrue(get_latest_script(self.first_version))

    def test_get_latest_script_doesnt_redownload_same_script(self):
        self.assertTrue(get_latest_script(self.first_version))
        self.assertFalse(get_latest_script(self.first_version))

    def test_get_latest_script_downloads_new_script(self):
        get_latest_script(self.first_version)

        # Update the script version
        script = os.path.join(config.WOOEY_TEST_SCRIPTS, 'versioned_script', 'v2.py')
        with open(script) as o:
            v2 = self.storage.save(self.filename_func('v2.py'), o)

        res = add_wooey_script(script_path=v2, script_name='test_versions')
        second_version = self.rename_script(res['script'])

        self.assertTrue(get_latest_script(second_version))


class TestCleanupDeadJobs(mixins.ScriptFactoryMixin, TestCase):
    def test_handles_unresponsive_workers(self):
        # Ensure that if we cannot connect to celery, we do nothing.
        with mock.patch('wooey.tasks.celery_app.control.inspect') as inspect_mock:
            running_job = factories.generate_job(self.translate_script)
            running_job.status = WooeyJob.RUNNING
            running_job.save()

            inspect_mock.return_value = mock.Mock(
                active=mock.Mock(
                    return_value=None,
                )
            )
            cleanup_dead_jobs()
            self.assertEqual(WooeyJob.objects.get(pk=running_job.id).status, WooeyJob.RUNNING)

    def test_cleans_up_dead_jobs(self):
        # Make a job that is running but not active, and a job that is running and active.
        dead_job = factories.generate_job(self.translate_script)
        dead_job.status = WooeyJob.RUNNING
        dead_job.save()
        active_job = factories.generate_job(self.translate_script)
        active_job.status = WooeyJob.RUNNING
        active_job.celery_id = 'celery-id'
        active_job.save()
        with mock.patch('wooey.tasks.celery_app.control.inspect') as inspect_mock:
            inspect_mock.return_value = mock.Mock(
                active=mock.Mock(
                    return_value={
                        'worker-id': [
                            {
                                'id': active_job.celery_id,
                            }
                        ]
                    },
                )
            )
            cleanup_dead_jobs()

            # Assert the dead job is updated
            self.assertEqual(WooeyJob.objects.get(pk=dead_job.id).status, WooeyJob.FAILED)
            self.assertEqual(WooeyJob.objects.get(pk=active_job.id).status, WooeyJob.RUNNING)
